/*============================================================================
*NOME: FAZ_MONITOR.C
*
*   Projeto: Monitores no Minix
*
*   Versao:  v1.00
*
*   Autores: Flavio Alexande da Rocha Magacho Filho -> 9416719
*            Juliana Carpes Imperial                -> 9714169
*            Silvia Carla Moreira                   -> 9416382
* 
*Descricao do modulo de implementacao
*      Objetivos:
*       - implementar as funcoes que realmente realizam as chamadas de sistema 
*         de monitores e
*       - implementar uma funcao auxiliar que retorna a posicao de um monitor
*         na fila de monitores dado o seu id.
*============================================================================*/
 
#include <time.h>
#include <stdlib.h>
#include "mm.h"
#include <signal.h> 
#include "faz_monitor.h"


static tpMonitor Monitores[ I_MAX_MONITORES ]    ;  /* lista de monitores */
static int       iNumMons                    = 0 ;  /* numero de monitores */


/*=============================================================================
*Nome da funcao: iFazer_mutex_create
*============================================================================*/

int iFazer_mutex_create( void ) {

   int        i     ;  /* contador auxiliar */
   static int iRand ;  /* numero randomico */    

/* testa se nao ha' monitores */
   if ( iNumMons == 0 ) {
      
   /* coloca toda a lista de monitores como nao valida */
      for ( i = 0 ; i < I_MAX_MONITORES ; i++ ) {
         Monitores[ i ].iValido = 0 ;
      } /* for */
       
   /* gera um numero randomico para o id do primeiro monitor */
      srand( time( NULL ) ) ;
      iRand = ( rand( ) % 100 ) + 1 ;

   } /* if */

/* procura monitor nao existente */
   for ( i = 0 ; i < I_MAX_MONITORES && Monitores[ i ].iValido == 1 ; i++ ) { }
   
/* testa se o numero de monitores existentes e' o maximo possivel */
   if ( i < I_MAX_MONITORES ) {
      
      Monitores[ i ].iProcesso = -1 ;     /* processo = -1, monitor vazio */
      Monitores[ i ].iValido = 1 ;        /* i-esimo monitor passa a existir */
      Monitores[ i ].iPresos = 0 ;        /* inicializa numero de presos */
      Monitores[ i ].iNumConds = 0 ;      /* inicializa numero de variaveis */
      iNumMons++ ;                        /* incrementa numero de monitores */

   /* atribui id randomico ao monitor */
      Monitores[ i ].iMonId = iRand * ( i + 1 ) ; 
      
      return Monitores[ i ].iMonId ;      /* retorna id do novo monitor */

   } else {
      return -1 ;  /* retorno -1: numero de monitores ja' e' o maximo */
   } /* if */
   
} /* iFazer_mutex_create */


/*=============================================================================
*Nome da funcao: iFazer_mutex_lock
*============================================================================*/

int iFazer_mutex_lock( void ) {

   int iLivre   , /* posicao livre na tabela de processos presos no monitor */
       iPosicao ; /* posicao do monitor ao qual se quer pedir o lock */
  
   iPosicao = iDarMonitor( mm_in.m1_i1 ) ; /* pega posicao do monitor */ 
   
   if ( iPosicao < 0 ) {  /* testa se a posicao do monitor e' valida */
      return -1 ;
   } /* if */
   
/* testa se ha' processo no monitor, se houver, bloqueia o qual esta' 
   tentando entrar */
   if ( Monitores[ iPosicao ].iProcesso != -1 ) {    
   
      iLivre = Monitores[ iPosicao ].iPresos ; /* pega uma posicao livre */   
   
   /* testa se o numero de processos presos e' maior que maximo permitido */ 
      if ( iLivre >= I_MAX_PRESOS ) {
         return -1 ;
      } /* if */
 
   /* se o numero de processos presos nao e' maximo e o monitor e' valido, vai
      bloquear o processo que quer entrar no monitor */
 
      Monitores[ iPosicao ].iPresos++ ; /* aumenta numero de presos */

   /* processo preso e' o que mandou a mensagem ao MM */
      Monitores[ iPosicao ].ProcPresos[ iLivre ] = who ;
     
      dont_reply = TRUE ; /* MM nao responde ao processo, ele fica preso */

   } else {
 
   /* monitor nao estava ocupado, entao agora vai ficar ocupado pelo processo
      que quer dar um lock nele */
      Monitores[ iPosicao ].iProcesso = who ;      

   } /* if */

   return 0 ;   

} /* iFazer_mutex_lock */


/*=============================================================================
*Nome da funcao: iFazer_mutex_unlock
*============================================================================*/

int iFazer_mutex_unlock( void ) {

   int i        ,  /* contador auxiliar */
       iPosicao ;  /* posicao do monitor ao qual se quer pedir o unlock */
         
   iPosicao = iDarMonitor( mm_in.m1_i1 ) ; /* pega posicao do monitor */

   if ( iPosicao < 0 ) {   /* testa se a posicao do monitor e' valida */
      return -1 ;
   } /* if */

/* testa se o processo que quer sair do monitor esta' dentro dele */
   if ( Monitores[ iPosicao ].iProcesso != who ) {
      return -1 ;
   } /* if */
   
/* o monitor e' valido e o processo que quer sair dele esta' la' dentro */

/* se nao ha' processos presos, o monitor vai ficar livre */
   if ( Monitores[ iPosicao ].iPresos == 0 ) {
      Monitores[ iPosicao ].iProcesso = -1 ;
      return 0 ;
   } /* if */
      
/* libera o primeiro processo preso na lista de processos presos do monitor , 
   fazendo com que o MM "responda" a mensagem recebida pela funcao
   mutex_lock para o processo cujo indice da tabela de processos presos e' 0 */
   reply( Monitores[ iPosicao ].ProcPresos[ 0 ], 1, 0, NIL_PTR ) ;

/* o processo que passa a deter o lock do monitor e' o primeiro que estava na 
   fila de processos de presos do monitor */
   Monitores[ iPosicao ].iProcesso = Monitores[ iPosicao ].ProcPresos[ 0 ] ;  

/* rearruma a lista de processos presos passando todos os processos para a 
   posicao anterior */
   for ( i = 0 ; i < Monitores[ iPosicao ].iPresos - 1 ; i++ ) {   
      Monitores[ iPosicao ].ProcPresos[ i ] 
               = Monitores[ iPosicao ].ProcPresos[ i + 1 ] ;
   } /* for */
         
   Monitores[ iPosicao ].iPresos-- ; /* decrementa numero de presos */
   return 0 ;

} /* iFazer_mutex_unlock */


/*=============================================================================
*Nome da funcao: iFazer_cond_create
*============================================================================*/

int iFazer_cond_create( void ) {
   
   int iPosicao ,  /* posicao do monitor que tera' nova variavel de condicao */
       iCondId  ;  /* posicao onde vai entrar a nova variavel de condicao */

   iPosicao = iDarMonitor( mm_in.m1_i1 ) ; /* pega posicao do monitor */

   if ( iPosicao < 0 ) { /* testa se posicao do monitor e' valida */
      return -1 ;
   } /* if */

/* testa se o numero de variaveis de condicao nao e' maior que o maximo 
   permitido */
   if ( Monitores[ iPosicao ].iNumConds < I_MAX_CONDS ) {
      
   /* o id da nova variavel de condicao e' o numero de variaveis de condicao */
      iCondId = Monitores[ iPosicao ].iNumConds ;

   /* o numero de processos presos na fila desta variavel e' zero */ 
      Monitores[ iPosicao ].VarConds[ iCondId ].iPresos = 0 ;

   /* aumenta o numero de variaveis de condicao do monitor */
      Monitores[ iPosicao ].iNumConds++ ;

      return iCondId ; /* retorna id da variavel de condicao */
    
   } else {
      return -1 ;  /* numero de variaveis chegou ao maximo permitido */
   } /* if */

} /* iFazer_cond_create */


/*=============================================================================
*Nome da funcao: iFazer_cond_wait
*============================================================================*/

int iFazer_cond_wait( void ) {
      
   int iLivre   , /* posicao livre na variavel de condicao do monitor */
       iPosicao , /* posicao do monitor ao qual se quer dar o wait */
       iCondId  , /* id da variavel de condicao onde o processo vai dar wait */
       i        ; /* contador auxiliar */ 

   iPosicao = iDarMonitor( mm_in.m1_i1 ) ;  /* pega posicao do monitor */
   iCondId = mm_in.m1_i2 ; /* id da variavel de condicao e' o da mensagem */
   
   if ( iPosicao < 0 ) { /* testa se a posicao do monitor e' valida */
      return -1 ;
   } /* if */

/* testa se o id da variavel de condicao e' valido */
   if ( iCondId >= I_MAX_CONDS 
     || iCondId < 0 ) {
     return -1 ;
   } /* if */

/* testa se o processo que quer dar se bloquear esta' dentro do monitor */
   if ( Monitores[ iPosicao ].iProcesso != who ) {
      return -1 ;
   } /* if */

/* pega uma posicao livre da variavel para bloquear o processo */
   iLivre = Monitores[ iPosicao ].VarConds[ iCondId ].iPresos ;  
   
/* testa se o numero de processos presos e' o maior que maximo permitido */ 
   if ( iLivre >= I_MAX_PRESOS ) {
      return -1 ;
   } /* if */
 
/* se o numero de processos presos na variavel de condicao nao e' maximo, o 
   monitor e a variavel de condicao sao validos e o processo que quer se 
   bloquear esta' dentro do monitor, o processo vai se bloquear */
   Monitores[ iPosicao ].VarConds[ iCondId ].ProcPresos[ iLivre ] = who ; 
   
/* aumenta numero de processos presos na variavel de condicao */
   Monitores[ iPosicao ].VarConds[ iCondId ].iPresos++ ;

   dont_reply = TRUE ; /* MM nao responde ao processo, ele fica preso */

/* se nao ha' processos presos, o monitor vai ficar livre */
   if ( Monitores[ iPosicao ].iPresos == 0 ) {
      Monitores[ iPosicao ].iProcesso = -1 ;
      return 0 ;
   } /* if */
      
/* libera o primeiro processo preso na lista de processos presos do monitor 
   iMonId, fazendo com que o MM "responda" a mensagem recebida pela funcao
   mutex_lock para o processo cujo indice da tabela de processos presos e' 0 */
   reply( Monitores[ iPosicao ].ProcPresos[ 0 ], 1, 0, NIL_PTR ) ;

/* o processo que passa a deter o lock do monitor e' o primeiro que estava na 
   fila de processos de presos do monitor */
   Monitores[ iPosicao ].iProcesso = Monitores[ iPosicao ].ProcPresos[ 0 ] ;  

/* rearruma a lista de processos presos passando todos os processos para a 
   posicao anterior */
   for ( i = 0 ; i < Monitores[ iPosicao ].iPresos - 1 ; i++ ) {   
      Monitores[ iPosicao ].ProcPresos[ i ] 
               = Monitores[ iPosicao ].ProcPresos[ i + 1 ] ;
   } /* for */
         
   Monitores[ iPosicao ].iPresos-- ; /* diminui o numero de processos presos */
   return 0 ;   

} /* iFazer_cond_wait */


/*=============================================================================
*Nome da funcao: iFazer_cond_notify
*============================================================================*/

int iFazer_cond_notify( void ) {
   
   int iLivre   , /* posicao livre na tabela de processos presos no monitor */
       iPosicao , /* posicao do monitor no qual se quer dar o notify */
       iCondId  , /* id da variavel de condicao no qual se quer dar notify */
       i        ; /* contador auxiliar */ 

   iPosicao = iDarMonitor( mm_in.m1_i1 ) ;  /* pega posicao do monitor */
   iCondId = mm_in.m1_i2 ; /* id da variavel de condicao e' o da mensagem */
   
   if ( iPosicao < 0 ) { /* testa se posicao do monitor e' valida */
     return -1 ;
   } /* if */

/* testa se o id da variavel de condicao e' valido */
   if ( iCondId >= I_MAX_CONDS 
     || iCondId < 0 ) {
     return -1 ;
   } /* if */

/* testa se o processo que quer dar notify esta' dentro do monitor */
   if ( Monitores[ iPosicao ].iProcesso != who ) {
      return -1 ;
   } /* if */

/* se nao ha' presos na fila da variavel de condicao, o notify e' perdido */
   if ( Monitores[ iPosicao ].VarConds[ iCondId ].iPresos == 0 ) {
      return 0 ;
   } /* if */

/* pega uma posicao livre da fila do monitor para bloquear o processo que vai 
   sair da fila da variavel de condicao do monitor */
   iLivre = Monitores[ iPosicao ].iPresos ;  
   
/* testa se o numero de processos presos e' o maior que maximo permitido */ 
   if ( iLivre >= I_MAX_PRESOS ) {
      return -1 ;
   } /* if */
 
/* se o numero de processos presos na fila do monitor nao e' maximo, o 
   monitor e a variavel de condicao sao validos e o processo que quer  
   desbloquear o primeiro a fila de condicao esta' dentro do monitor, 
   o processo vai de conseguir */

/* o primeiro da fila da variavel de condicao vai para o fim da fila do monitor
   */ 
   Monitores[ iPosicao ].ProcPresos[ iLivre ] 
            = Monitores[ iPosicao ].VarConds[ iCondId ].ProcPresos[ 0 ] ; 
   
/* aumenta numero de processos presos na fila do monitor */
   Monitores[ iPosicao ].iPresos++ ;

/* rearruma a lista de processos presos na variavel de condicao passando todos 
   os processos para a posicao anterior */
   for ( i = 0 ; 
         i < Monitores[ iPosicao ].VarConds[ iCondId ].iPresos - 1 ; 
         i++ ) {   
      Monitores[ iPosicao ].VarConds[ iCondId ].ProcPresos[ i ] 
              = Monitores[ iPosicao ].VarConds[ iCondId ].ProcPresos[ i + 1 ] ;
   } /* for */
         
/* diminui o numero de processos presos na fila da variavel de condicao */
   Monitores[ iPosicao ].VarConds[ iCondId ].iPresos-- ;

   return 0 ;
 
} /* iFazer_cond_notify */


/*=============================================================================
*Nome da funcao: iFazer_mutex_destroy
*============================================================================*/

int iFazer_mutex_destroy( void ) {
   
   int iPosicao , /* posicao do monitor que se quer destruir */
       i        ; /* contador auxiliar */

   iPosicao = iDarMonitor( mm_in.m1_i1 ) ;  /* pega posicao do monitor */   

/* testa se o id do monitor e' valido */
   if ( iPosicao < 0 ) {
     return -1 ;
   } /* if */

/* se ha' algum processo dentro do monitor, ele nao pode ser destruido */
   if ( Monitores[ iPosicao ].iProcesso != -1 ) {
      return -1 ;
   } /* if */ 

/* se ha' processos presos, ele nao pode ser destruido */
   if ( Monitores[ iPosicao ].iPresos != 0 ) {
      return -1 ;
   } /* if */

/* se ha' processos presos em alguma das filas de condicao do monitor, ele nao
   pode ser destruido */
   for ( i = 0 ; i < Monitores[ iPosicao ].iNumConds ; i++ ) {
      if ( Monitores[ iPosicao ].VarConds[ i ].iPresos > 0 ) {
         return -1 ;
      } /* if */
   } /* for */
   
   Monitores[ iPosicao ].iValido = 0 ; /* o monitor passa a nao existir mais */
   iNumMons-- ;                        /* decrementa numero de monitores */
   return 0 ;

} /* iFazer_mutex_destroy */


/*=============================================================================
*Nome da funcao: iDarMonitor
*============================================================================*/

int iDarMonitor( int iMonId ) {
    
   int i ; /* contador auxiliar */

/* procura na lista de monitores o monitor com o id iMonId */
   for ( i = 0 ; i < I_MAX_MONITORES ; i++ ) {

   /* o id deve ser igual a iMonId e o monitor deve ser valido */
      if ( Monitores[ i ].iMonId == iMonId 
        && Monitores[ i ].iValido == 1 ) {
         return i ;
      
      } /* if */
   
   } /* for */

   return -1 ; /* o monitor de id iMonId nao foi encontrado na lista */
 
} /* iDarMonitor */


/* faz_monitor.c */