/*============================================================================
*NOME: FAZ_MONITOR.H
*
*   Projeto: Monitores no Minix
*
*   Versao:  v1.00
*
*   Autores: Flavio Alexande da Rocha Magacho Filho -> 9416719
*            Juliana Carpes Imperial                -> 9714169
*            Silvia Carla Moreira                   -> 9416382
* 
*Descricao do modulo de declaracao
*      Objetivos:
*      - definir as funcoes que realmente realizam as chamadas de sistema de 
*        monitores e
*      - definir uma funcao que devolve o indice do monitor dado o seu id.
*============================================================================*/
 

#if !defined( faz_monitor_MOD )
#define faz_monitor_MOD

#define I_MAX_CONDS     50   /* maximo de variaveis de condicao */
#define I_MAX_MONITORES 50   /* maximo de monitores */
#define I_MAX_PRESOS    100  /* maximo de processos bloqueados */


/*=============================================================================
*NOME: tpCond
*Descricao do tipo: 
*    representa uma variavel de condicao.
*
*Campos:
*    - iPresos   : numero de processos presos na fila da variavel de condicao e
*    - ProcPresos: um vetor com os ids dos processos presos.
*============================================================================*/

typedef struct {
   int iPresos                    ,
       ProcPresos[ I_MAX_PRESOS ] ;
} tpCond ;    


/*=============================================================================
*NOME: tpMonitor
*Descricao do tipo: 
*    representa um monitor.
*
*Campos:
*    - iMonId    : id do monitor;
*    - iPresos   : numero de processos presos na fila do monitor;
*    - iProcesso : id do processo que esta' dentro do monitor;
*    - iValido   : indica se o monitor existe;
*    - iNumConds : numero de variaveis de condicao do monitor;
*    - ProcPresos: um vetor com os ids dos processos presos na fila do 
*                  monitor e
*    - VarConds  : um vetor de variaveis de condicao do monitor.
*============================================================================*/

typedef struct {
   int    iMonId                     , 
          iPresos                    ,
          iProcesso                  ,
          iValido                    ,
          iNumConds                  ,
          ProcPresos[ I_MAX_PRESOS ] ;
   tpCond VarConds[ I_MAX_CONDS ]    ;
} tpMonitor ;


/*=============================================================================
*Nome da funcao: iFazer_mutex_create
*Descricao da funcao: cria uma estrutura correspondente a um monitor.
*
*Valor retornado:
*   -> retorna o identificador do monitor criado ou
*   -> -1 se nao foi possivel criar o novo monitor.
*============================================================================*/

int iFazer_mutex_create( void ) ;


/*=============================================================================
*Nome da funcao: iFazer_mutex_lock
*Descricao da funcao: permitir que um processo entre dentro de um dado 
*                     monitor.
*
*Valor retornado:
*   -1 se monitor nao existe ou limite para processos presos estorou.
*============================================================================*/

int iFazer_mutex_lock( void ) ;


/*=============================================================================
*Nome da funcao: iFazer_mutex_unlock
*Descricao da funcao: permitir que um processo saia de dentro de um dado 
*                     monitor.
*
*Valor retornado:
*    -1 se monitor nao existe ou o processo corrente nao possui o lock do
*    monitor.
*============================================================================*/

int iFazer_mutex_unlock( void ) ;


/*=============================================================================
*Nome da funcao: iFazer_cond_create
*Descricao da funcao: cria uma variavel de condicao associada a um dado
*                     monitor.
*
*Valor retornado:
*    -> id da variavel de condicao ou
*    -> -1 se o monitor nao existe ou um limite do sistema foi atingido.
*============================================================================*/

int iFazer_cond_create( void ) ;


/*=============================================================================
*Nome da funcao: iFazer_cond_wait
*Descricao da funcao: bloquear o processo corrente na fila da variavel de 
*                     condicao dada de um dado monitor.
*
*Valor retornado:
*   -1 se monitor ou variavel de condicao nao existem ou processo nao possui o
*   lock do monitor ou o limite de processos presos na variavel de condicao
*   estorou.
*============================================================================*/

int iFazer_cond_wait( void ) ;


/*=============================================================================
*Nome da funcao: iFazer_cond_notify
*Descricao da funcao: desbloquear o primeiro processo da fila da variavel de 
*                     condicao dada de um dado monitor.
*
*Valor retornado:
*   -1 se monitor ou variavel de condicao nao existem ou processo nao possui o
*   lock do monitor ou o limite de processos presos na fila do monitor estorou.
*============================================================================*/

int iFazer_cond_notify( void ) ;


/*=============================================================================
*Nome da funcao: iFazer_mutex_destroy
*Descricao da funcao: destroi um dado monitor, liberando os recursos alocados
*                     para ele.
*
*Valor retornado:
*    -1 se o monitor nao existe ou ha' processos bloqueados em variaveis de
*    condicao dele.
*============================================================================*/

int iFazer_mutex_destroy( void ) ;


/*=============================================================================
*Nome da funcao: iDarMonitor
*Descricao da funcao: definir uma funcao que devolve o indice do monitor dado 
*                     o seu ID.
*
*Parametro recebido:
*    - iMonId: ID do monitor.
*
*Valor retornado:
*    - o indice dele no vetor de monitores ou -1 se id do monitor nao existe.
*============================================================================*/

int iDarMonitor( int iMonId ) ;


#endif

/* faz_monitor.h */

