/*============================================================================
*NOME: MONITOR.C
*
*   Projeto: Monitores no Minix
*
*   Versao:  v1.00
*
*   Autores: Flavio Alexande da Rocha Magacho Filho -> 9416719
*            Juliana Carpes Imperial                -> 9714169
*            Silvia Carla Moreira                   -> 9416382
* 
*Descricao do modulo de implementacao
*      Objetivo:
*         Implementar chamadas de sistema que implementem monitores no MINIX.
*============================================================================*/
 

#include <lib.h>
#include <monitor.h>


/*=============================================================================
*Nome da funcao: mutex_create
*============================================================================*/

int mutex_create( void ) {
   
   message Msg      ;  /* mensagem a ser enviada a MM */
   int     iSucesso ;  /* variavel que indica se a chamada foi bem sucedida */
   
/* envia a mensagem a MM, se foi possivel criar o monitor com sucesso, retorna 
   o ID do monitor criado, senao, retorna -1 */
   iSucesso = _syscall( MM, I_MUTEX_CREATE, &Msg ) ;   
   if ( iSucesso < 0 ) {
      return - 1 ;
   } else {
      return iSucesso ;
   } /* if */

} /* mutex_create */


/*=============================================================================
*Nome da funcao: mutex_lock
*============================================================================*/

int mutex_lock( int monid ) {

   message Msg      ;  /* mensagem a ser enviada a MM */
   int     iSucesso ;  /* variavel que indica se a chamada foi bem sucedida */

   Msg.m1_i1 = monid ;  /* coloca ID do monitor na mensagem */

/* envia a mensagem a MM, se nao foi possivel dar o lock no monitor com 
   sucesso, retorna -1 */
   iSucesso = _syscall( MM, I_MUTEX_LOCK, &Msg ) ;   
   if ( iSucesso < 0 ) {
      return - 1 ;
   } else {
      return iSucesso ;
   } /* if */

} /* mutex_lock */


/*=============================================================================
*Nome da funcao: mutex_unlock
*============================================================================*/

int mutex_unlock( int monid ) {
   
   message Msg      ;  /* mensagem a ser enviada a MM */
   int     iSucesso ;  /* variavel que indica se a chamada foi bem sucedida */
   
   Msg.m1_i1 = monid ;  /* coloca ID do monitor na mensagem */

/* envia a mensagem a MM, se nao foi possivel dar o unlock no monitor com 
   sucesso, retorna -1 */
   iSucesso = _syscall( MM, I_MUTEX_UNLOCK, &Msg ) ;   
   if ( iSucesso < 0 ) {
      return - 1 ;
   } else {
      return iSucesso ;
   } /* if */

} /* mutex_unlock */


/*=============================================================================
*Nome da funcao: cond_create
*============================================================================*/

int cond_create( int monid ) {
   
   message Msg      ;  /* mensagem a ser enviada a MM */
   int     iSucesso ;  /* variavel que indica se a chamada foi bem sucedida */
   
   Msg.m1_i1 = monid ;  /* coloca ID do monitor na mensagem */

/* envia a mensagem a MM, se nao foi possivel criar a variavel de condicao 
   com sucesso, retorna -1, caso contrario retorna o ID dela */
   iSucesso = _syscall( MM, I_COND_CREATE, &Msg ) ;   
   if ( iSucesso < 0 ) {
      return - 1 ;
   } else {
      return iSucesso ;
   } /* if */

} /* cond_create */


/*=============================================================================
*Nome da funcao: cond_wait
*============================================================================*/

int cond_wait( int monid ,
               int condid ) {
  
   message Msg      ;  /* mensagem a ser enviada a MM */
   int     iSucesso ;  /* variavel que indica se a chamada foi bem sucedida */
   
   Msg.m1_i1 = monid ;  /* coloca ID do monitor na mensagem */
   Msg.m1_i2 = condid ;  /* coloca ID da variavel de condicao na mensagem */

/* envia a mensagem a MM, se nao foi possivel dar o wait na variavel de 
   condicao com sucesso, retorna -1 */ 
   iSucesso = _syscall( MM, I_COND_WAIT, &Msg ) ;   
   if ( iSucesso < 0 ) {
      return - 1 ;
   } else {
      return iSucesso ;
   } /* if */

} /* cond_wait */


/*=============================================================================
*Nome da funcao: cond_notify
*============================================================================*/

int cond_notify( int monid ,
                 int condid ) {
  
   message Msg      ;  /* mensagem a ser enviada a MM */
   int     iSucesso ;  /* variavel que indica se a chamada foi bem sucedida */
   
   Msg.m1_i1 = monid ;   /* coloca ID do monitor na mensagem */
   Msg.m1_i2 = condid ;  /* coloca ID da variavel de condicao na mensagem */

/* envia a mensagem a MM, se nao foi possivel dar o notify na variavel de 
   condicao com sucesso, retorna -1 */
   iSucesso = _syscall( MM, I_COND_NOTIFY, &Msg ) ;   
   if ( iSucesso < 0 ) {
      return - 1 ;
   } else {
      return iSucesso ;
   } /* if */
       
} /* cond_notify */


/*=============================================================================
*Nome da funcao: mutex_destroy
*============================================================================*/

int mutex_destroy( int monid ) {
     
   message  Msg     ;  /* mensagem a ser enviada a MM */
   int     iSucesso ;  /* variavel que indica se a chamada foi bem sucedida */
   
   Msg.m1_i1 = monid ;  /* coloca ID do monitor na mensagem */

/* envia a mensagem a MM, se nao foi possivel destruir o monitor com sucesso, 
   retorna -1 */
   iSucesso = _syscall( MM, I_MUTEX_DESTROY, &Msg ) ;   
   if ( iSucesso < 0 ) {
      return - 1 ;
   } else {
      return iSucesso ;
   } /* if */
  
} /* mutex_destroy */


/* monitor.c */





