/*============================================================================
*NOME: MONITOR.H
*
*   Projeto: Monitores no Minix
*
*   Versao:  v1.00
*
*   Autores: Flavio Alexande da Rocha Magacho Filho -> 9416719
*            Juliana Carpes Imperial                -> 9714169
*            Silvia Carla Moreira                   -> 9416382
* 
*Descricao do modulo de declaracao
*      Objetivo:
*         Definir chamadas de sistema para implementar monitores no MINIX.
*============================================================================*/
 

#if !defined( monitor_MOD )
#define monitor_MOD

/* definicoes para as chamadas de monitores */
#define I_MUTEX_CREATE    77  /* criacao de monitor */
#define I_MUTEX_LOCK      78  /* lock de monitor */
#define I_MUTEX_UNLOCK    79  /* unlock de monitor */
#define I_COND_CREATE     80  /* criacao de variavel de condicao */
#define I_COND_WAIT       81  /* espera em fila de variavel de condicao */
#define I_COND_NOTIFY     82  /* libera primeiro da fila de condicao */
#define I_MUTEX_DESTROY   83  /* destroi monitor */


/*=============================================================================
*Nome da funcao: mutex_create
*Descricao da funcao: cria uma estrutura correspondente a um monitor.
*
*Valor retornado:
*   -> retorna o identificador do monitor criado ou
*   -> -1 se nao foi possivel criar o novo monitor.
*============================================================================*/

int mutex_create( void ) ;


/*=============================================================================
*Nome da funcao: mutex_lock
*Descricao da funcao: permitir que um processo entre dentro de um dado 
*                     monitor.
*
*Parametro recebido:
*    monid - o id do monitor.
*
*Valor retornado:
*   -1 se monitor nao existe ou a fila do monitor esta cheia.
*============================================================================*/

int mutex_lock( int monid ) ;


/*=============================================================================
*Nome da funcao: mutex_unlock
*Descricao da funcao: permitir que um processo saia de dentro de um dado 
*                     monitor.
*
*Parametro recebido:
*    monid - o id do monitor.
*
*Valor retornado:
*    -1 se monitor nao existe ou o processo corrente nao possui o lock do
*    monitor.
*============================================================================*/

int mutex_unlock( int monid ) ;


/*=============================================================================
*Nome da funcao: cond_create
*Descricao da funcao: cria uma variavel de condicao associada a um dado
*                     monitor.
*
*Parametro recebido:
*    monid - o id do monitor.
*
*Valor retornado:
*    -> id da variavel de condicao ou
*    -> -1 se o monitor nao existe ou um limite do sistema foi atingido.
*============================================================================*/

int cond_create( int monid ) ;


/*=============================================================================
*Nome da funcao: cond_wait
*Descricao da funcao: bloquear o processo corrente na fila da variavel de 
*                     condicao dada de um dado monitor.
*
*Parametros recebidos:
*    monid  - o id do monitor e
*    condid - o id da variavel de condicao.
*
*Valor retornado:
*   -1 se monitor ou variavel de condicao nao existem ou processo nao possui o
*   lock do monitor ou fila da variavel de condicao esta' cheia.
*============================================================================*/

int cond_wait( int monid ,
               int condid ) ;


/*=============================================================================
*Nome da funcao: cond_notify
*Descricao da funcao: desbloquear o primeiro processo da fila da variavel de 
*                     condicao dada de um dado monitor.
*
*Parametros recebidos:
*    monid  - o id do monitor e
*    condid - o id da variavel de condicao.
*
*Valor retornado:
*   -1 se monitor ou variavel de condicao nao existem ou processo nao possui o
*   lock do monitor ou fila do monitor esta' cheia.
*============================================================================*/

int cond_notify( int monid ,
                 int condid ) ;


/*=============================================================================
*Nome da funcao: mutex_destroy
*Descricao da funcao: destroi um dado monitor, liberando os recursos alocados
*                     para ele.
*
*Parametro recebido:
*    monid - o id do monitor.
*
*Valor retornado:
*    -1 se o monitor nao existe ou ha' processos bloqueados em variaveis de
*    condicao dele.
*============================================================================*/

int mutex_destroy( int monid ) ;


#endif

/* monitor.h */








